rm(list=ls())
library(BB)
library(mvtnorm)
library(mnormt)
ini.time <- Sys.time()
source("twostep.r")
n<-1e4;
beta0  <- c(rep(1/2, 7))
d <- length(beta0)
corr  <- 0.5
sigmax  <- matrix(corr, d, d) + diag(1-corr, d)
set.seed(0)
rpt <- 1000

case <- 1
r.values <- c(300, 500, 800, 1200)
r.loc <- as.numeric(Sys.getenv("PBS_ARRAY_INDEX"))
r <- r.values[r.loc]
cat(case, r, "\n")
filename <- paste("output/fixr", case, "r", r, ".Rdata", sep="")
switch(case,
       {    ## 1:  normal mean 0
           X  <- rmvnorm(n, rep(0, d), sigmax)
       }, { ## 2: mixnormal means 1 and -1
           X  <- rmvnorm(n/2, rep(1, d), sigmax/4)
           X  <- rbind(rmvnorm(n/2, rep(-1, d), sigmax/4), X)
       }, { ## 3: t3
           X  <- rmt(n, rep(0, d), sigmax, 3) / 10
       }, { ## 4: exponential
           X <- matrix(rexp(n*d, 2), n, d)
       }, { ## 5: normal mean 1 variance 1
           X  <- rmvnorm(n, rep(1.5, d), sigmax)
       }, { ## 6: normal unequal variance
           sig <- diag(1 / (1:7))
           sigU <- sig %*% sigmax %*% sig
           X  <- rmvnorm(n, rep(0, d), sigU)
       ## }, { ## lognormal
       ##     filename <- "output/04.Rdata"
       ##     X  <- rmvnorm(n, rep(0, d), sigmax)
       ##     X <- exp(X) / 10
       })
P  <- 1 - 1 / (1 + exp(X %*% beta0))
Y  <- rbinom(n, 1, P)
print(mean(Y))
## 

## r.ss <- c(100, 200, 300, 500, 700, 1000)
r.ss <- round(seq(100, r-50, length=20))

lrs <- length(r.ss)
Beta.simp <- matrix(NA, d, rpt*lrs)
nm <- 6
Beta.opt <- matrix(NA, d*nm, rpt*lrs)
itr <- 0

for (r1 in r.ss) {
r2 <- r - r1
cat(r1, r2, r, " ");
set.seed(10)
for (rr in 1:rpt) {
    if (rr%/%10 == rr/10) cat(rr, "-")
    itr <- itr+1
    fit.alg.simp <- AlgTwoStp(r1+r2, 0)
    Beta.simp[,itr] <- fit.alg.simp$simp
    if (any(fit.alg.simp$msg != "Successful convergence")) {
        warning(paste(rr, "not converge", fit.alg$msg.simp))
        Beta.simp[,itr] <- NA
    }
    fit.alg <- AlgTwoStp(r1, r2)
    if (any(fit.alg$msg != "Successful convergence")) {
        warning(paste(rr, "not converge", fit.alg$msg))
        next
    }
    Beta.opt[,itr] <- c(fit.alg$opt)
}
cat("\n")
}

print(Sys.time() - ini.time)

l2.simp <- (Beta.simp - beta0)^2
l2.opt  <- (Beta.opt  - beta0)^2
mse.simp <- mse.opt <- c()
for (i in 1:lrs) {
    loc <- ((i-1)*rpt+1):(i*rpt)
    mse.simp <- c(mse.simp, sum(rowMeans(l2.simp[,loc], na.rm=TRUE)))
    mse.opt <- rbind(mse.opt, colSums(matrix(
        rowMeans(l2.opt[,loc], na.rm=TRUE), d, nm)))
}

rec <- cbind(mse.simp, mse.opt)
save.image(file=filename)

plot(r.ss, rec[,1]/rec[,1], xlab="", ylab="", pch="1",
     ylim=c(min(rec/rec[,1]), max(rec/rec[,1])), type="o")
pp <- dim(rec)[2]
for(i in 2:pp)
    lines(r.ss, rec[,i]/rec[,1], type="o", pch=paste(i), lty=i, col=i)
legend("center", lty=1:pp, pch=paste(1:pp), col=1:pp,
       legend=c("uniform", "MV", "MEV", "MVC", "MEVC", "Li", "L0"))

print(rec)
print(sum(rec))
