## getMLE <- function(x, y, w) {
##     fn <- function(beta, X=x, Y=y, W=w) {
##         pr  <- 1 - 1 / (1 + exp(X %*% beta))
##         -sum((Y * log(pr) + (1 - Y) * log(1 - pr)) * W)
##     }
##     BBoptim(rep(0, d), fn, quiet=T, control=list(trace=F))
## }

## x=x.simp; y=y.simp; w=pinv.simp

getMLE <- function(x, y, w) {
    beta <- rep(0, d)
    loop  <- 1
    Loop  <- 100
    msg <- "NA"
    while (loop <= Loop) {
        pr <- c(1 - 1 / (1 + exp(x %*% beta)))
        H <- t(x) %*% (pr * (1 - pr) * w * x)
        S <- colSums((y - pr) * w * x)
        tryCatch(
            {shs <- NA
             shs <- solve(H, S) }, 
            error=function(e){
                cat("\n ERROR :", loop, conditionMessage(e), "\n")})
        if (is.na(shs[1])) {
            msg <- "Not converge"
            beta <- loop <- NA
            break
        }
        beta.new <- beta + shs
        tlr  <- sum((beta.new - beta)^2)
        beta  <- beta.new
        if(tlr < 0.000001) {
            msg <- "Successful convergence"
            break
        }
        if (loop == Loop)
            warning("Maximum iteration reached")
        loop  <- loop + 1
    }
    list(par=beta, message=msg, iter=loop)
}

## r1 <- r1+r; r2=0

AlgTwoStp <- function(r1=r1, r2=r2) {
    if (r2 == 0) {
        idx.simp <- sample(1:n, r1, T)
        x.simp <- X[idx.simp,]
        y.simp <- Y[idx.simp]
        pinv.simp <- n
        fit.simp <- getMLE(x=x.simp, y=y.simp, w=pinv.simp)
        beta.simp <- fit.simp$par
        msg <- fit.simp$message
        return(list(simp=beta.simp, msg=msg))
    }
    if (r2 != 0) {
        n1 <- sum(Y)
        n0 <- n - n1
        PI.prop <- rep(1/(2*n0), n)
        PI.prop[Y==1] <- 1/(2*n1)
        idx.prop <- sample(1:n, r1, T, PI.prop)
        x.prop <- X[idx.prop,]
        y.prop <- Y[idx.prop]
        pinv.prop <- n
        pinv.prop <- 1/PI.prop[idx.prop]
        fit.prop <- getMLE(x=x.prop, y=y.prop, w=pinv.prop)
        beta.prop <- fit.prop$par
        if (is.na(beta.prop[1]))
            return(list(opt=NA, msg="first stage not converge"))
        P.prop  <- 1 - 1 / (1 + exp(X %*% beta.prop))
        ## ## MEVC
        ## PI.optA <- sqrt(P.prop * (1 - P.prop) * rowSums(X^2))
        ## PI.optA <- PI.optA / sum(PI.optA)
        ## idx.optA <- sample(1:n, r2, T, PI.optA)
        ## x.optA <- X[c(idx.optA, idx.prop),]
        ## y.optA <- Y[c(idx.optA, idx.prop)]
        ## pinv.optA <- c(1 / PI.optA[idx.optA], pinv.prop)
        ## fit.optA <- getMLE(x=x.optA, y=y.optA, w=pinv.optA)

        ## mVc
        PI.mVc <- sqrt((Y - P.prop)^2 * rowSums(X^2))
        PI.mVc <- PI.mVc / sum(PI.mVc)
        idx.mVc <- sample(1:n, r2, T, PI.mVc)
        x.mVc <- X[c(idx.mVc, idx.prop),]
        y.mVc <- Y[c(idx.mVc, idx.prop)]
        pinv.mVc <- c(1 / PI.mVc[idx.mVc], pinv.prop)
        fit.mVc <- getMLE(x=x.mVc, y=y.mVc, w=pinv.mVc)
        ## mMSE
        p.prop <- P.prop[idx.prop]
        w.prop <- p.prop * (1 - p.prop)
        W.prop <- solve(t(x.prop) %*% (x.prop * w.prop * pinv.prop))
        PI.mMSE <- sqrt((Y - P.prop)^2 * rowSums((X%*%W.prop)^2))
        PI.mMSE <- PI.mMSE / sum(PI.mMSE)
        idx.mMSE <- sample(1:n, r2, T, PI.mMSE)
        x.mMSE <- X[c(idx.mMSE, idx.prop),]
        y.mMSE <- Y[c(idx.mMSE, idx.prop)]
        pinv.mMSE <- c(1 / PI.mMSE[idx.mMSE], pinv.prop)
        fit.mMSE <- getMLE(x=x.mMSE, y=y.mMSE, w=pinv.mMSE)
        ## ## MEV
        ## PI.optEV <- sqrt(P.prop*(1-P.prop) * rowSums((X%*%W.prop)^2))
        ## PI.optEV <- PI.optEV / sum(PI.optEV)
        ## idx.optEV <- sample(1:n, r2, T, PI.optEV)
        ## x.optEV <- X[c(idx.optEV, idx.prop),]
        ## y.optEV <- Y[c(idx.optEV, idx.prop)]
        ## pinv.optEV <- c(1 / PI.optEV[idx.optEV], pinv.prop)
        ## fit.optEV <- getMLE(x=x.optEV, y=y.optEV, w=pinv.optEV)

        ## LCC
        PI.LCC <- abs(Y - P.prop)
        PI.LCC <- PI.LCC / sum(PI.LCC)
        idx.LCC <- sample(1:n, r2, T, PI.LCC)
        x.LCC <- X[c(idx.LCC),]
        y.LCC <- Y[c(idx.LCC)]
        pinv.LCC <- 1
        fit.LCC <- getMLE(x=x.LCC, y=y.LCC, w=pinv.LCC)

        ## PI.optU <- abs(Y - P.prop)
        ## PI.optU <- PI.optU / sum(PI.optU)
        ## idx.optU <- sample(1:n, r2, T, PI.optU)
        ## x.optU <- X[c(idx.optU, idx.prop),]
        ## y.optU <- Y[c(idx.optU, idx.prop)]
        ## pinv.optU <- c(1 / PI.optU[idx.optU], pinv.prop)
        ## fit.optU <- getMLE(x=x.optU, y=y.optU, w=pinv.optU)
        
        ## PI.optM <- sqrt((Y - P.prop)^2 * apply(X^2, 1, max))
        ## PI.optM <- PI.optM / sum(PI.optM)
        ## idx.optM <- sample(1:n, r2, T, PI.optM)
        ## x.optM <- X[c(idx.optM, idx.prop),]
        ## y.optM <- Y[c(idx.optM, idx.prop)]
        ## pinv.optM <- c(1 / PI.optM[idx.optM], pinv.prop)
        ## fit.optM <- getMLE(x=x.optM, y=y.optM, w=pinv.optM)

        opt <- cbind(fit.mMSE$par, fit.mVc$par, fit.LCC$par+beta.prop)
        msg <- c(fit.prop$message, fit.mMSE$message, 
                 fit.mVc$message, fit.LCC$message)
        ## opt <- cbind(fit.mMSE$par, fit.optEV$par, fit.mVc$par,
        ##              fit.optA$par, fit.optM$par, fit.LCC$par+beta.prop)
        ## msg <- c(fit.prop$message, fit.mMSE$message, fit.optEV$message,
        ##          fit.mVc$message,fit.optA$message, 
        ##          fit.optM$message, fit.LCC$message)
        return(list(opt=opt, msg=msg))
    }
}


## i <- 0
## while (i <= 10) {
##     i <- i + 1
##     tryCatch(
##         {
##             if (i==7) stop("Urgh, the iphone is in the blender !")
##             print(i)
##         }, 
##         error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
## }

## robustLog <- function(x) {
##     tryCatch(log(x),
##              warning = function(w) {print(paste("negative argument", x)); 
##                                     log(-x)},
##              error = function(e) {print(paste("non-numeric argument", x)); 
##                                   NaN}) 
## }
