rm(list=ls())
## library(BB)
library(mvtnorm)
library(mnormt)
ini.time <- Sys.time()
source("twostep.r")
n<-1e4;
beta0  <- c(rep(1/2, 7))
d <- length(beta0)
corr  <- 0.5
sigmax  <- matrix(corr, d, d) + diag(1-corr, d)
rpt <- 1000

case <- 1
case <- as.numeric(Sys.getenv("PBS_ARRAY_INDEX"))
print(case)
filename <- paste("output/", case, ".Rdata", sep="")
set.seed(0)
switch(case,
       {    ## 1:  normal mean 0
           X  <- rmvnorm(n, rep(-2.14, d), sigmax)
       }, { ## 2: normal mean 1 variance 1
           X  <- rmvnorm(n, rep(-2.9, d), sigmax)
       ## }, { ## 3: normal unequal variance
       ##     sig <- diag(1 / (1:7))
       ##     sigU <- sig %*% sigmax %*% sig
       ##     X  <- rmvnorm(n, rep(0, d), sigU)
       ## }, { ## 4: mixnormal means 1 and -1
       ##     X  <- rmvnorm(n/2, rep(1, d), sigmax/4)
       ##     X  <- rbind(rmvnorm(n/2, rep(-1, d), sigmax/4), X)
       ## }, { ## 5: t3
       ##     X  <- rmt(n, rep(0, d), sigmax, 3) / 10
       ## }, { ## 6: exponential
       ##     X <- matrix(rexp(n*d, 2), n, d)
       ## }, { ## lognormal
       ##     filename <- "output/04.Rdata"
       ##     X  <- rmvnorm(n, rep(0, d), sigmax)
       ##     X <- exp(X) / 10
       })
P  <- 1 - 1 / (1 + exp(X %*% beta0))
Y  <- rbinom(n, 1, P)

print(mean(Y))
##

fit.full <- getMLE(X, Y, 1)
## print(system.time({
Beta.full <- matrix(NA, d, rpt)
## full data Bootstrap
for (bb in 1:rpt) {
    idx.full <- sample(1:n, n, T)
    x.full <- X[idx.full,]
    y.full <- Y[idx.full]
    Beta.full[,bb] <- getMLE(x.full, y.full, 1)$par
}
mse.full <- sum(rowMeans((Beta.full - beta0)^2, , na.rm=TRUE))
## mse.full <- 0

## P.full <- 1 - 1 / (1 + exp(c(X %*% fit.full$par)))
## w.full <- P.full * (1 - P.full)
## W.full <- solve(t(X) %*% (X * w.full))
## sum(diag(solve(W.full)))


r1 <- 200
r <- 200

## r1.ss <- round(seq(100, r, length=20))
r.ss <- c(100, 200, 300, 500, 700, 1000)## [5:6]
## r.ss <- c(500, 700, 1000)## [5:6]
lrs <- length(r.ss)
## Beta.simp <- matrix(NA, d, rpt*lrs)
nm <- 4
Beta.opt <- matrix(NA, d*nm, rpt*lrs)
itr <- 0

for (r in r.ss) {
cat(r, " ");
set.seed(0)
for (rr in 1:rpt) {
    if (rr%/%100 == rr/100) cat(rr, "-")
    itr <- itr+1
    ## fit.alg.simp <- 0## AlgTwoStp(r1+r, 0)
    ## Beta.simp[,itr] <-0 ## fit.alg.simp$simp
    ## if (any(fit.alg.simp$msg != "Successful convergence")) {
    ##     warning(paste(rr, "not converge", fit.alg.simp$msg))
    ##     Beta.simp[,itr] <- NA
    ## }
    fit.alg <- AlgTwoStp(r1, r)
    if (any(fit.alg$msg != "Successful convergence")) {
        warning(paste(rr, "not converge:", fit.alg$msg))
        next
    }
    Beta.opt[,itr] <- c(fit.alg$opt)
}
cat("\n\n")
}

print(Sys.time() - ini.time)

Beta.simp <- matrix(NA, d, rpt*lrs)
ncc <- c()
itr <- 0
for (r in r.ss) {
cat(r, " ");
nc <- 0
set.seed(0)
for (rr in 1:rpt) {
    if (rr%/%100 == rr/100) cat(rr, "-")
    itr <- itr+1
    fit.alg.simp <- AlgTwoStp(r1+r, 0)
Beta.simp[,itr] <- fit.alg.simp$simp
    if (any(fit.alg.simp$msg != "Successful convergence")) {
        nc = nc + 1
    }
}
ncc <- c(ncc, nc)
cat("\n\n")
}
print(ncc)

l2.simp <- (Beta.simp - beta0)^2
l2.opt  <- (Beta.opt  - beta0)^2
mse.simp <- mse.opt <- c()
for (i in 1:lrs) {
    loc <- ((i-1)*rpt+1):(i*rpt)
    mse.simp <- c(mse.simp, sum(rowMeans(l2.simp[,loc], na.rm=TRUE)))
    mse.opt <- rbind(mse.opt, colSums(matrix(
        rowMeans(l2.opt[,loc], na.rm=TRUE), d, nm)))
}

rec <- cbind(mse.simp, mse.opt)

plot(r.ss, rec[,1], xlab="", ylab="", pch="1",
     ylim=c(0, 4), type="o")
pp <- dim(rec)[2]
for(i in 2:pp)
    lines(r.ss, rec[,i], type="o", pch=paste(i), lty=i, col=i)
abline(h=mse.full)
legend("center", lty=1:pp, pch=paste(1:pp), col=1:pp,
       legend=c("uniform", "MV", "MVC", "LCC", "CC"))

print(rec)
print(sum(rec))
print(mean(Y))

save.image(file=filename)
print(Sys.time() - ini.time)
