rm(list=ls())
ini.time <- Sys.time()
source("twostep.r")

set.seed(0)
rpt <- 1000

case <- 1
case <- as.numeric(Sys.getenv("PBS_ARRAY_INDEX"))
print(case)
filename <- paste("output/", case, ".Rdata", sep="")
load("SUSY_X.Rdata")
load("SUSY_Y.Rdata")

## Use all features
ini.time <- Sys.time()
X <- t(t(X) - colMeans(X))
X <- t(t(X) / apply(X, 2, sd))
X <- cbind(1, X)
n <- dim(X)[1]
X.t <- X[-(1:(0.9*n)),]
Y.t <- Y[-(1:(0.9*n))]
X <- X[1:(0.9*n),]
Y <- Y[1:(0.9*n)]
n <- dim(X)[1]
d <- dim(X)[2]
print(mean(Y))
## 

print(system.time({fit.full <- getMLE(X, Y, 1)}))
beta.full <- fit.full$par
print(beta.full)

P.full  <- 1 - 1 / (1 + exp(c(X %*% beta.full)))
w.full <- P.full * (1 - P.full)
W.full <- solve(t(X) %*% (X * w.full))
sum(diag(W.full))

se.full <- sqrt(diag(W.full))
print(round(se.full, 3))

set.seed(0)
r1 <- 200
r <- 200

## r1.ss <- round(seq(100, r, length=20))
r.ss <- c(100, 200, 300, 500, 700, 1000)[case]
lrs <- length(r.ss)
Beta.simp <- matrix(NA, d, rpt*lrs)
nm <- 2
Beta.opt <- matrix(NA, d*nm, rpt*lrs)
itr <- 0
SE.simp <- matrix(NA, d, rpt*lrs)
SE.opt <- matrix(NA, d*nm, rpt*lrs)

ini.time <- Sys.time()
r <- r.ss
## for (r in r.ss) {
cat(r, " ");
set.seed(10)
for (rr in (1:rpt)) {
    if (rr%/%100 == rr/100) cat(rr, "-")
    itr <- itr+1
    fit.alg.simp <- AlgTwoStp(r1+r, 0)
    Beta.simp[,itr] <- fit.alg.simp$simp
    SE.simp[,itr] <- fit.alg.simp$amse
    if (any(fit.alg.simp$msg != "Successful convergence")) {
        warning(paste(rr, "not converge", fit.alg.simp$msg))
        Beta.simp[,itr] <- NA
    }
    fit.alg <- AlgTwoStp(r1, r)
    if (any(fit.alg$msg != "Successful convergence")) {
        warning(paste(rr, "not converge:", fit.alg$msg))
        next
    }
    Beta.opt[,itr] <- c(fit.alg$opt)
    SE.opt[,itr] <- c(fit.alg$amse)
}
cat("\n")
## save(list=ls()[-which(ls()=="X"|ls()=="Y")] ,file=filename)
## }
print(Sys.time() - ini.time)

l2.simp <- (Beta.simp - beta.full)^2
l2.opt  <- (Beta.opt  - beta.full)^2
mse.simp <- mse.opt <- c()
for (i in 1:lrs) {
    loc <- ((i-1)*rpt+1):(i*rpt)
    mse.simp <- c(mse.simp, sum(rowMeans(l2.simp[,loc], na.rm=TRUE)))
    mse.opt <- rbind(mse.opt, colSums(matrix(
        rowMeans(l2.opt[,loc], na.rm=TRUE), d, nm)))
}
rec <- cbind(mse.simp, mse.opt)

se.simp <- se.opt <- c()
for (i in 1:lrs) {
    loc <- ((i-1)*rpt+1):(i*rpt)
    se.simp <- c(se.simp, sum(rowMeans(SE.simp[,loc], na.rm=TRUE)))
    se.opt <- rbind(se.opt, colSums(matrix(
        rowMeans(SE.opt[,loc], na.rm=TRUE), d, nm)))
}
se.rec <- cbind(se.simp, se.opt)


plot(r.ss, rec[,1], xlab="", ylab="", pch="1",
     ylim=c(min(rec), max(rec)), type="o")
pp <- dim(rec)[2]
for(i in 2:pp)
    lines(r.ss, rec[,i], type="o", pch=paste(i), lty=i, col=i)
legend("center", lty=1:pp, pch=paste(1:pp), col=1:pp,
       legend=c("uniform", "mMSE", "mVc"))

print(rec)
print(sum(rec))

rm(list=c("X","Y","l2.opt", "l2.simp", "loc", "P.full", "w.full"))
save.image(file=paste("output/c", case, ".Rdata", sep=""))
