library(kulife)
rm(list=ls())
ini.time <- Sys.time()
source("twostep.r")

load("SUSY_X.Rdata")
load("SUSY_Y.Rdata")

## Use all features
ini.time <- Sys.time()
X <- t(t(X) - colMeans(X))
X <- t(t(X) / apply(X, 2, sd))
X <- cbind(1, X)
n <- dim(X)[1]
X.t <- X[-(1:(0.9*n)),]
Y.t <- Y[-(1:(0.9*n))]
X <- X[1:(0.9*n),]
Y <- Y[1:(0.9*n)]
n <- dim(X)[1]
d <- dim(X)[2]
nm <- 2

print(system.time({fit.full <- getMLE(X, Y, 1)}))
beta.full <- fit.full$par
## beta.full <- rep(0, d)
## print(beta.full)

r1 <- 200
r <- 800
set.seed(0)
rec <- c()
Beta.simp <- Beta.opt <- c()

rpt <- 1000
rec.area <- c()
for(rr in 1:rpt) {
fit.alg <- AlgTwoStp(r1, r)
beta.opt <- fit.alg$opt
fit.alg.simp <- AlgTwoStp(r1+r, 0)
beta.simp <- fit.alg.simp$simp


Pxb.simp <- X.t %*% beta.simp
Pxb.full <- X.t %*% beta.full
Pxb.opt  <- X.t %*% beta.opt

Tn <- Tp <- c()
cut.points <- seq(0, 1, length=21)[-c(1, 21)]
ctp <- log(cut.points / (1-cut.points))
lctp <- length(ctp)
for (p.t in ctp) {
    tn.simp <- sum(2 * Y.t + (Pxb.simp>p.t) == 0)
    tp.simp <- sum(2 * Y.t + (Pxb.simp>p.t) == 3)
    tn.full <- sum(2 * Y.t + (Pxb.full>p.t) == 0)
    tp.full <- sum(2 * Y.t + (Pxb.full>p.t) == 3)
    tn.opt  <- colSums(2*Y.t+(Pxb.opt>p.t)==0)
    tp.opt  <- colSums(2*Y.t+(Pxb.opt>p.t)==3)
    Tn <- cbind(Tn, c(tn.simp, tn.opt, tn.full))
    Tp <- cbind(Tp, c(tp.simp, tp.opt, tp.full))
}

n1.t <- sum(Y.t)
n0.t <- length(Y.t) - n1.t
TN <- cbind(0, Tn, n0.t) / n0.t
TP <- cbind(n1.t, Tp, 0) / n1.t
ltp <- dim(TP)[1]
area <- c()
for (i in 1:ltp)
    area <- c(area, auc(TN[i,], TP[i,], from=0, to=1))
rec.area <- cbind(rec.area, area)
}
print(rowMeans(rec.area))
plot(TP[1,], TN[1,], type="l", xlab="True positive", ylab="Ture negative")
for (i in 1:ltp)
    lines(TP[i,], TN[i,], col=i, lty=i)
mtext("True positive", side=1, line=1.8)
mtext("True negative", side=2, line=1.8)

timetorun <- Sys.time() - ini.time
rm(list=c("X", "Y", "X.t", "Y.t", "Pxb.opt", "Pxb.simp", "Pxb.full"))

areaout <- cbind(rowMeans(rec.area), apply(rec.area, 1, sd))
write.csv(areaout, file="output/area.csv")

save.image(file="output/singlerun.Rdata")
print(Sys.time() - ini.time)

## rm(list=ls())
## load("single.Rdata")
## rm(list=c("Pxb.full", "Pxb.opt", "Pxb.simp"))
## save.image("single.Rdata")
