getMLE <- function(x, y, w) {
    beta <- rep(0, d)
    loop  <- 1
    Loop  <- 100
    msg <- "NA"
    while (loop <= Loop) {
        pr <- c(1 - 1 / (1 + exp(x %*% beta)))
        H <- t(x) %*% (pr * (1 - pr) * w * x)
        S <- colSums((y - pr) * w * x)
        tryCatch(
            {shs <- NA
             shs <- solve(H, S) }, 
            error=function(e){
                cat("\n ERROR :", loop, conditionMessage(e), "\n")})
        if (is.na(shs[1])) {
            msg <- "Not converge"
            beta <- loop <- NA
            break
        }
        beta.new <- beta + shs
        tlr  <- sum((beta.new - beta)^2)
        beta  <- beta.new
        if(tlr < 0.000001) {
            msg <- "Successful convergence"
            break
        }
        if (loop == Loop)
            warning("Maximum iteration reached")
        loop  <- loop + 1
    }
    list(par=beta, message=msg, iter=loop)
}

AlgTwoStp <- function(r1=r1, r2=r2) {
    if (r2 == 0) {
        idx.simp <- sample(1:n, r1, T)
        x.simp <- X[idx.simp,]
        y.simp <- Y[idx.simp]
        pinv.simp <- n
        fit.simp <- getMLE(x=x.simp, y=y.simp, w=pinv.simp)
        beta.simp <- fit.simp$par
        msg <- fit.simp$message
if (fit.simp$message == "Successful convergence") {
p.simp  <- 1 - 1 / (1 + exp(c(x.simp %*% beta.simp)))
w.simp <- p.simp * (1 - p.simp)
W.simp <- solve(t(x.simp) %*% (x.simp * (w.simp * n))) * r1 * n
Vc.simp <- t(x.simp) %*% (x.simp * (y.simp-p.simp)^2 * n^2) / r1^2 / n^2
V.simp <- W.simp %*% Vc.simp %*% W.simp
amse.simp <- sqrt(diag(V.simp))
}
else
amse.simp = NA
        return(list(simp=beta.simp, msg=msg, amse=amse.simp))
    }
    if (r2 != 0) {
        n1 <- sum(Y)
        n0 <- n - n1
        PI.prop <- rep(1/(2*n0), n)
        PI.prop[Y==1] <- 1/(2*n1)
        idx.prop <- sample(1:n, r1, T, PI.prop)
        x.prop <- X[idx.prop,]
        y.prop <- Y[idx.prop]
        pinv.prop <- n
        pinv.prop <- 1/PI.prop[idx.prop]
        fit.prop <- getMLE(x=x.prop, y=y.prop, w=pinv.prop)
        beta.prop <- fit.prop$par
        if (is.na(beta.prop[1]))
            return(list(opt=NA, msg="first stage not converge"))
        P.prop  <- 1 - 1 / (1 + exp(X %*% beta.prop))
        ## mVC
        PI.mVc <- sqrt((Y - P.prop)^2 * rowSums(X^2))
        PI.mVc <- PI.mVc / sum(PI.mVc)
        idx.mVc <- sample(1:n, r2, T, PI.mVc)
        x.mVc <- X[c(idx.mVc, idx.prop
                      ),]
        y.mVc <- Y[c(idx.mVc, idx.prop
                      )]
        pinv.mVc <- c(1 / PI.mVc[idx.mVc], pinv.prop
                       )
        fit.mVc <- getMLE(x=x.mVc, y=y.mVc, w=pinv.mVc)

ru <- length(y.mVc)
beta.mVc <- fit.mVc$par
p.mVc  <- 1 - 1 / (1 + exp(c(x.mVc %*% beta.mVc)))
w.mVc <- p.mVc * (1 - p.mVc)
W.mVc <- solve(t(x.mVc) %*% (x.mVc * (w.mVc * pinv.mVc))) * ru * n
Vc.mVc <- t(x.mVc) %*% (x.mVc * (y.mVc-p.mVc)^2 * pinv.mVc^2) / ru^2 / n^2
V.mVc <- W.mVc %*% Vc.mVc %*% W.mVc

        ## mMSE
        p.prop <- P.prop[idx.prop]
        w.prop <- p.prop * (1 - p.prop)
        W.prop <- solve(t(x.prop) %*% (x.prop * w.prop * pinv.prop))
        PI.mMSE <- sqrt((Y - P.prop)^2 * rowSums((X%*%W.prop)^2))
        PI.mMSE <- PI.mMSE / sum(PI.mMSE)
        idx.mMSE <- sample(1:n, r2, T, PI.mMSE)
        x.mMSE <- X[c(idx.mMSE, idx.prop
                      ),]
        y.mMSE <- Y[c(idx.mMSE, idx.prop
                      )]
        pinv.mMSE <- c(1 / PI.mMSE[idx.mMSE], pinv.prop
                       )
        fit.mMSE <- getMLE(x=x.mMSE, y=y.mMSE, w=pinv.mMSE)

ru <- length(y.mMSE)
beta.mMSE <- fit.mMSE$par
p.mMSE  <- 1 - 1 / (1 + exp(c(x.mMSE %*% beta.mMSE)))
w.mMSE <- p.mMSE * (1 - p.mMSE)
W.mMSE <- solve(t(x.mMSE) %*% (x.mMSE * (w.mMSE * pinv.mMSE))) * ru * n
Vc.mMSE <- t(x.mMSE) %*% (x.mMSE * (y.mMSE-p.mMSE)^2 * pinv.mMSE^2) / ru^2 / n^2
V.mMSE <- W.mMSE %*% Vc.mMSE %*% W.mMSE

        opt <- cbind(beta.mMSE, beta.mVc)
        amse <- cbind(sqrt(diag(V.mMSE)), sqrt(diag(V.mVc)))
        msg <- c(fit.prop$message, fit.mMSE$message, fit.mVc$message)
        return(list(opt=opt, msg=msg, amse=amse))
    }
}
